import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
const _excluded = ["tonal", "color", "name", "values", "width", "refs", "rects", "updateDefs", "updateLegend", "minMax", "animate", "animateTimeout", "PathProps", "BackgroundProps", "BorderProps", "LegendItemProps", "className"];
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
import React from 'react';
import { is, percentageFromValueWithinRange, valueFromPercentageWithinRange } from '@amaui/utils';
import { classNames, style as styleMethod, useAmauiTheme } from '@amaui/style-react';
import PathElement from '../Path';
import LineElement from '../Line';
import TypeElement from '../Type';
import useMediaQuery from '../useMediaQuery';
import { staticClassName, valueBreakpoints } from '../utils';
const useStyle = styleMethod(theme => ({
  root: {},
  legend_item: {
    userSelect: 'none'
  },
  legend_icon: {
    width: '4px',
    height: '12px'
  }
}), {
  name: 'amaui-ColumnChartItem'
});
const ColumnChartItem = /*#__PURE__*/React.forwardRef((props_, ref) => {
  const theme = useAmauiTheme();
  const props = React.useMemo(() => _objectSpread(_objectSpread(_objectSpread({}, theme?.ui?.elements?.all?.props?.default), theme?.ui?.elements?.amauiColumnChartItem?.props?.default), props_), [props_]);
  const Line = React.useMemo(() => theme?.elements?.Line || LineElement, [theme]);
  const Path = React.useMemo(() => theme?.elements?.Path || PathElement, [theme]);
  const Type = React.useMemo(() => theme?.elements?.Type || TypeElement, [theme]);
  const {
      tonal = true,
      color = 'primary',
      name,
      values,
      width: widthItem = 14,
      refs: refs_,
      rects,
      updateDefs,
      updateLegend,
      minMax,
      animate: animate_,
      animateTimeout: animateTimeout_,
      PathProps,
      BackgroundProps,
      BorderProps,
      LegendItemProps,
      className
    } = props,
    other = _objectWithoutProperties(props, _excluded);
  const {
    classes
  } = useStyle();
  const refs = {
    minMax: React.useRef(),
    theme: React.useRef(),
    path: React.useRef(),
    pathStyle: React.useRef({}),
    animate: React.useRef(),
    animateTimeout: React.useRef(),
    init: React.useRef()
  };
  const breakpoints = {};
  theme.breakpoints.keys.forEach(key => {
    if (theme.breakpoints.media[key]) breakpoints[key] = useMediaQuery(theme.breakpoints.media[key], {
      element: refs.path.current
    });
  });
  const animate = valueBreakpoints(animate_, true, breakpoints, theme);
  const animateTimeout = valueBreakpoints(animateTimeout_, 140, breakpoints, theme);
  const [value, setValue] = React.useState();
  const [init, setInit] = React.useState();
  refs.theme.current = theme;
  refs.animate.current = animate;
  refs.animateTimeout.current = animateTimeout;
  refs.init.current = init;
  const LegendItem = React.useCallback(legendItemProps => {
    return /*#__PURE__*/React.createElement(Line, _extends({
      gap: 1,
      direction: "row",
      align: "center"
    }, legendItemProps, LegendItemProps, {
      className: classNames([staticClassName('ColumnChartItem', theme) && ['amaui-ColumnChartItem-legend-item'], legendItemProps?.className, LegendItemProps?.className, classes.legend_item])
    }), /*#__PURE__*/React.createElement("span", {
      className: classNames([staticClassName('ColumnChartItem', theme) && ['amaui-ColumnChartItem-legend-icon'], classes.legend_icon]),
      style: {
        background: !refs.theme.current.palette.color[color] ? color : refs.theme.current.palette.color[color]['main']
      }
    }), /*#__PURE__*/React.createElement(Type, {
      version: "b3"
    }, name));
  }, [theme]);
  const make = () => {
    // Make values into x, y, coordinates
    // normalized in rect width, height values

    // invert y so 0, 0 is at bottom left
    if (rects?.wrapper && values) {
      const {
        width,
        height
      } = rects.wrapper;
      const [x, y] = values;
      const values_ = {
        x: percentageFromValueWithinRange(x, minMax.min.x, minMax.max.x),
        y: percentageFromValueWithinRange(y, minMax.min.y, minMax.max.y)
      };
      values_.x = valueFromPercentageWithinRange(values_.x, 0, width);
      values_.y = valueFromPercentageWithinRange(values_.y, 0, height);
      const value_ = [values_.x, height - values_.y].map(item_ => Math.abs(item_));
      const element = /*#__PURE__*/React.createElement("g", null, /*#__PURE__*/React.createElement(Path, _extends({
        x: value_[0] - widthItem / 2,
        y: value_[1],
        width: widthItem,
        height: height - value_[1],
        fill: !refs.theme.current.palette.color[color] ? color : refs.theme.current.palette.color[color]['main'],
        stroke: "none",
        Component: "rect"
      }, PathProps, {
        style: _objectSpread(_objectSpread(_objectSpread({}, PathProps?.style), refs.animate.current && refs.init.current !== 'animated' && {
          opacity: 0,
          transform: 'scaleY(0)'
        }), {}, {
          transformOrigin: 'bottom'
        }, refs.pathStyle.current)
      })));

      // Legend item
      if (is('function', updateLegend)) {
        updateLegend(previous => {
          const newValues = [...(previous || [])];
          if (!newValues.find(itemLegend => itemLegend.item?.name === name)) {
            newValues.push({
              item: {
                name,
                values
              },
              element: /*#__PURE__*/React.createElement(LegendItem, null)
            });
          }
          return newValues;
        });
      }

      // Value
      setValue(element);
    }
  };
  const initMethod = React.useCallback(() => {
    if (animate) {
      if (!init) {
        refs.pathStyle.current = {
          transition: theme.methods.transitions.make(['transform', 'opacity'], {
            timing_function: 'decelerated'
          })
        };
        setInit(true);
        setTimeout(() => {
          refs.pathStyle.current = _objectSpread(_objectSpread({}, refs.pathStyle.current), {}, {
            opacity: 1,
            transform: 'scaleY(1)'
          });
          setInit('animated');
        }, refs.animateTimeout.current);
      }
    }
  }, [init, animate]);
  React.useEffect(() => {
    make();
    initMethod();
  }, [values, theme, animate, init, !!refs.path.current]);
  React.useEffect(() => {
    make();
  }, [rects?.wrapper]);
  return value && /*#__PURE__*/React.cloneElement(value, _objectSpread({
    className
  }, other));
});
ColumnChartItem.displayName = 'amaui-ColumnChartItem';
export default ColumnChartItem;