import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
const _excluded = ["tonal", "color", "version", "underline", "Component", "className", "style", "children"];
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
import React from 'react';
import { classNames, style as styleMethod, useAmauiTheme } from '@amaui/style-react';
import TypeElement from '../Type';
import { staticClassName } from '../utils';
const useStyle = styleMethod(theme => ({
  root: {
    position: 'relative',
    display: 'inline-flex',
    transition: theme.methods.transitions.make(['color', 'text-decoration'], {
      duration: 'xxs'
    }),
    '&::before': {
      content: "''",
      display: 'inline-flex',
      position: 'absolute',
      top: '50%',
      left: '50%',
      transform: 'translate(-50%, -50%)',
      background: 'currentColor',
      width: '100%',
      height: '100%',
      zIndex: '0',
      padding: '0.14em 0.77em',
      borderRadius: `${theme.shape.radius.unit / 8 * 0.4}em`,
      transition: theme.methods.transitions.make(['opacity', 'background']),
      opacity: '0'
    }
  },
  focus: {
    '&::before': {
      opacity: theme.palette.visual_contrast.default.opacity.selected
    }
  },
  // Underline
  underline: {
    textDecoration: 'underline'
  },
  underline_hover: {
    '&:hover': {
      textDecoration: 'underline'
    }
  },
  // Color
  color_default: {
    color: theme.palette.text.default.primary
  },
  color_neutral: {
    color: theme.palette.color.neutral.main
  },
  color_primary: {
    color: theme.palette.color.primary.main
  },
  color_secondary: {
    color: theme.palette.color.secondary.main
  },
  color_tertiary: {
    color: theme.palette.color.tertiary.main
  },
  color_quaternary: {
    color: theme.palette.color.quaternary.main
  },
  color_info: {
    color: theme.palette.color.info.main
  },
  color_success: {
    color: theme.palette.color.success.main
  },
  color_warning: {
    color: theme.palette.color.warning.main
  },
  color_error: {
    color: theme.palette.color.error.main
  },
  // Tonal
  tonal_color_neutral: {
    color: theme.methods.palette.color.value('neutral', 30)
  },
  tonal_color_primary: {
    color: theme.methods.palette.color.value('primary', 30)
  },
  tonal_color_secondary: {
    color: theme.methods.palette.color.value('secondary', 30)
  },
  tonal_color_tertiary: {
    color: theme.methods.palette.color.value('tertiary', 30)
  },
  tonal_color_quaternary: {
    color: theme.methods.palette.color.value('quaternary', 30)
  },
  tonal_color_info: {
    color: theme.methods.palette.color.value('info', 30)
  },
  tonal_color_success: {
    color: theme.methods.palette.color.value('success', 30)
  },
  tonal_color_warning: {
    color: theme.methods.palette.color.value('warning', 30)
  },
  tonal_color_error: {
    color: theme.methods.palette.color.value('error', 30)
  }
}), {
  name: 'amaui-Link'
});
const Link = /*#__PURE__*/React.forwardRef((props_, ref) => {
  const theme = useAmauiTheme();
  const props = React.useMemo(() => _objectSpread(_objectSpread(_objectSpread({}, theme?.ui?.elements?.all?.props?.default), theme?.ui?.elements?.amauiLink?.props?.default), props_), [props_]);
  const Type = React.useMemo(() => theme?.elements?.Type || TypeElement, [theme]);
  const {
      tonal = true,
      color: color_ = 'primary',
      version = 'b2',
      underline = true,
      Component = 'a',
      className,
      style,
      children
    } = props,
    other = _objectWithoutProperties(props, _excluded);
  const {
    classes
  } = useStyle();
  const [focus, setFocus] = React.useState(false);
  const [hover, setHover] = React.useState(false);
  const styles = {
    root: {}
  };
  const onFocus = React.useCallback(event => {
    setFocus(true);
  }, []);
  const onBlur = React.useCallback(event => {
    setFocus(false);
  }, []);
  const onMouseEnter = React.useCallback(event => {
    setHover(true);
  }, []);
  const onMouseLeave = React.useCallback(event => {
    setHover(false);
  }, []);
  const color = color_;
  let palette;
  if (!classes[`color_${color}`]) {
    styles.root.color = color;
    palette = theme.methods.color(color);
    if (tonal) styles.root.color = theme.methods.palette.color.value(undefined, 30, true, palette);
  }
  if (underline === true && !hover) {
    styles.root.textDecorationColor = theme.methods.palette.color.value(color, 70, true, palette);
  }
  if (hover) {
    styles.root.color = theme.methods.palette.color.value(color, 10, true, palette);
  }
  return /*#__PURE__*/React.createElement(Type, _extends({
    ref: ref,
    onFocus: onFocus,
    onBlur: onBlur,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave,
    onTouchStart: onMouseEnter,
    onTouchEnd: onMouseLeave,
    version: version,
    role: "link",
    Component: Component,
    className: classNames([staticClassName('Link', theme) && ['amaui-Link-root', hover && `amaui-Link-hover`, focus && `amaui-Link-focus`], className, classes.root, classes[`color_${color}`], tonal && classes[`tonal_color_${color}`], underline === true && classes.underline, underline === 'hover' && classes.underline_hover, focus && classes.focus]),
    style: _objectSpread(_objectSpread({}, style), styles.root)
  }, other), children);
});
Link.displayName = 'amaui-Link';
export default Link;